from Bio import SeqIO

def GC_content(sequence):
    G_count = sequence.count('G')
    C_count = sequence.count('C')
    return (G_count + C_count)/len(sequence)

def form_bin_array(bin_dir, collected_bins, bin_array, contig_id_list):
    # bin_dir: bin_directory inside binny;
    # collected_bins: bin_file name;
    # bin_array: the empty list containing len(contig_id) elements.
    # contig_id_list: the list of contigs with their id.
    
    for i in collected_bins:
        bin_path = str(bin_dir) + "/" + i 
        individual_bin = list(SeqIO.parse(bin_path, "fasta"))
        for elem in individual_bin:
            index_in_list = contig_id_list.index(elem.id)
            bin_name = i.replace(".fasta","")
            # print(bin_name)
            bin_array[index_in_list] = bin_name
    return 

def form_marker_array(empty_marker_array, annot_file, contig_id):
    with open(annot_file) as af:
        annot_lines = af.readlines()
    marker_dict = {}
    default = "no markers in the database"
    for i in range(len(annot_lines)):
        i_th_ORF = annot_lines[i].split("\t")
        
        key = i_th_ORF[0]
        ORF = i_th_ORF[-1]
        ORF_split = ORF.strip().split(";")
        # Initiate the gathering.
        if ("checkm_marker" in ORF_split[-1]) and (key not in marker_dict.keys()):
            marker_dict[key] = ""
            marker_dict[key] += ORF_split[-1].split("=")[-1]
        elif ("checkm_marker" in ORF_split[-1]) and (key in marker_dict.keys()):
            marker_dict[key] += ","
            marker_dict[key] += ORF_split[-1].split("=")[-1]
        else:
            if key not in marker_dict.keys():
                marker_dict[key] = ""
            else:
                continue
    
    for i in marker_dict.items():
        key = i[0]
        value = i[-1]
        index = contig_id.index(key)
        empty_marker_array[index] = value

    return empty_marker_array

def form_depth_array(empty_depth_array, contig_id, depth_file):
    with open(depth_file) as df:
        depth_lines = df.readlines()
    for i in range(len(depth_lines)):
        contig, depth = depth_lines[i].strip().split("\t",1)[0], depth_lines[i].strip().split("\t",1)[1]
        if contig in contig_id:
            empty_depth_array[contig_id.index(contig)] = depth
    return empty_depth_array

def store_basic_info(contig_id_array, contig_seq_array, contig_len_array, contig_gc_array, assembly_contigs):
    # simply change all of the files
    for i in assembly_contigs:
        contig_id_array.append(i.id)
        contig_seq_array.append(i.seq)
        contig_len_array.append(len(i.seq))
        contig_gc_array.append(GC_content(str(i.seq)))
    return

def find_finest_taxon(id_hierarchy, all_organisms):
    i = 0
    length_id_hierarchy = len(id_hierarchy)
    while i < length_id_hierarchy:
        try:
            the_organism = all_organisms[id_hierarchy[i]]
            i+=1
        except KeyError:
            break
    return all_organisms[id_hierarchy[i-1]]

def all_organisms_dict(names_dump):
    all_organisms = dict()
    with open(names_dump, "r") as nd:
        org_lines = nd.readlines()
    
    for i in org_lines:
        i = i.strip().replace("\t", "")
        org_line = i.split("|")
        if "scientific name" in org_line:
            all_organisms[org_line[0]] = org_line[1]
    return all_organisms

def form_contig_taxa_array(empty_taxon_array, contig_id, names_dump, contig_annotation):
    with open(contig_annotation, "r") as contig_f:
        contig_taxonomy_lines = contig_f.readlines()

    info_contig_taxonomy = contig_taxonomy_lines[1:]
    contig2taxid=dict()
    for i in info_contig_taxonomy:
        line_info = i.split("\t")
        if line_info[1] == "no taxid assigned":
            contig2taxid[line_info[0]] = "0"
        else:
            contig2taxid[line_info[0]] = line_info[-2]
    
    all_organisms = all_organisms_dict(names_dump)

    for i in contig2taxid.keys():
        index_contig = contig_id.index(i)
        if "*" in contig2taxid[i]:
            taxon_i = contig2taxid[i].replace("*","")
        taxon_i = contig2taxid[i]
        if taxon_i == "0":
            empty_taxon_array[index_contig] = "not classified"
        else:
            taxon_hierarchy = taxon_i.split(";")
            finest_classification = find_finest_taxon(taxon_hierarchy, all_organisms)
            empty_taxon_array[index_contig] = finest_classification

    return


import pandas as pd
import click

@click.command()
@click.option("--cat_prediction_txt", type=click.Path(exists=True))
@click.option("--output", type=click.STRING)
# @click.option("--abundance", type=click.STRING)

def main(cat_prediction_txt, output):
    # transfer it into a pandas dataframe.
    cat_df=pd.read_csv(cat_prediction_txt, sep="\t")
    copy_df = cat_df.loc[cat_df['lineage'].notna()]
    lineage_hierarchy = copy_df['lineage']
    finest_lineage = []
    for i in lineage_hierarchy:
        indi_hierarchy = i.replace("*","").split(";")
        finest_classification = indi_hierarchy[-1]
        # print(finest_classification)
        finest_lineage.append(finest_classification)
    
    copy_df['intake for krona'] = finest_lineage
    intake_dict = dict()
    for i in finest_lineage:
        if i in intake_dict.keys():
            intake_dict[i]+=1
        else:
            intake_dict[i]=1
    

# Here, only counts of contigs were used as magnitude, but that is arbitrarily trivial
# So what we need to do is try to get the real abundance of the sequences/taxonomy to work. 
    counts = intake_dict.values()
    taxon_id = intake_dict.keys()
    krona_intake = pd.DataFrame()
    krona_intake['count'] = counts
    krona_intake['taxon id'] = taxon_id

    # add the magnitude of contigs by its depth
    
    krona_intake.to_csv(output, sep="\t", index=False)
    ##firstly, filter those without a ORF hit, retain those having it.
    return

if __name__ == "__main__":
    main()

    # Tips for changing the output for our krona tools:
    # 1. Shifting the count of taxonomy to the count of contig magnitudes + the official format of krona tools to work on.
    # 2. try to get the magnitude for each contig and then go for it. 
    # 3. Here is the problem: If you only use count of taxa found, it gives you how precise corgi did to classify each contig.
    #    But, adding the magnitude of contigs by its depth, can adjust the abundance to make it precise. 


from pathlib import Path
import typer
from snk_cli import CLI

chloroscan = CLI(Path(__file__).parent.parent)

@chloroscan.app.command()
def github():
    """ Launch the ChloroScan GitHub page. """
    typer.launch("https://github.com/Andyargueasae/chloroscan")


@chloroscan.app.command()
def docs():
    """ Launch the ChloroScan documentation. """
    typer.launch("https://andyargueasae.github.io/chloroscan/")



import Bio
from Bio import SeqIO
import os
from os import listdir
import pandas as pd
import numpy as np
import sys
import logging
from pathlib import Path


def refine_bins_renaming(seqs, cross_ref_df):
    taxonomy_dict = dict()
    seq_ids = [i.id for i in seqs]
    print(seq_ids)
    for seq_id in seq_ids:
        the_row = cross_ref_df.loc[cross_ref_df['contig id'] == seq_id]
        the_taxon = the_row['Taxon per Contig'].item()
        the_length = the_row['contig length'].item()
        if the_taxon not in taxonomy_dict.keys():
            taxonomy_dict[the_taxon] = 0
        taxonomy_dict[the_taxon] += the_length
    
    print(taxonomy_dict)
    sorted_dict_items = sorted(taxonomy_dict.items(), key = lambda x:x[1])
    print(sorted_dict_items)
    taxon_with_longest_length = sorted_dict_items[-1][0]
    return taxon_with_longest_length

output_dir_for_refined_bins = snakemake.output[0]
output_summary_info = snakemake.output[1]


Path(output_summary_info).touch()
# Inputs and params.

cross_reference_table = snakemake.input['cross_ref']
original_bins = snakemake.input['original_bins']+"/bins"
CAT_prediction = snakemake.input['CAT_prediction'] + "/out.CAT.contig2classification.txt"
BACTERIA_ID = snakemake.params['BACTERIA_ID']
EUKARYA_ID = "2759"
ROOT_ID = "1"
log_file = snakemake.log[0]
logging.basicConfig(
    filename=log_file,  # Specify the log file
    filemode='a',                # 'a' for append mode, 'w' for write (overwrite)
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',  # Log format
    level=logging.INFO           # Set the minimum log level to INFO
)

logger = logging.getLogger(__name__)

os.mkdir(output_dir_for_refined_bins)

if (os.stat(cross_reference_table).st_size == 0):
    print("Empty cross reference table, exit.\n")
    sys.exit(0)

cross_ref_refine_df = pd.read_csv(cross_reference_table,sep="\t", index_col=0)
bin_list = listdir(original_bins)
orig_CAT_df = pd.read_csv(CAT_prediction, sep="\t")
lineages = orig_CAT_df['lineage']

list_of_seqrec = dict()
for i in bin_list:
    filename = original_bins + "/" + i
    list_of_seqrec[filename] = list(SeqIO.parse(filename, "fasta"))


# First, identify which contig is bacterial and has no marker genes on it.
refined_contigs_dict = dict()

to_be_modified_id = []
for i in list_of_seqrec.keys():
    logging.info(f"Inspect bin {i.split('/')[-1]} contig composition.")
    contig_comp = list_of_seqrec[i]
    contig_ids = [i.id for i in contig_comp]
    contig_length = [len(i.seq) for i in contig_comp]
    logging.info(f"contig composition before refinement: {','.join(contig_ids)}\n")
    logging.info(f"Number of contigs: {len(contig_ids)}")
    logging.info(f"Summed Length of MAG: {sum(contig_length)}")
    # Length of MAG should be reported.
    for elem in contig_comp:
        elem_id = elem.id
        elem_in_crossref = cross_ref_refine_df.loc[cross_ref_refine_df['contig id'] == elem_id]
        elem_row = orig_CAT_df.loc[orig_CAT_df["# contig"] == elem.id]
        elem_lineage = np.array(elem_row['lineage'])[0]
        bacteria_taxonomy_id = BACTERIA_ID
        if not pd.isna(elem_lineage):
            elem_lineage_hierarchy = elem_lineage.split(";")
            logging.info("CAT/BAT taxonomic assignment of contig {}: {}".format(elem_id, elem_lineage_hierarchy))
            logging.info("Marker genes found on the contig: {}".format(np.array(elem_in_crossref['markers on the contig'])[0]))
            if (bacteria_taxonomy_id in elem_lineage_hierarchy) and (pd.isna(np.array(elem_in_crossref['markers on the contig'])[0])):
                logging.info("contig id: {} This is a bacterial contamination.".format(elem_id))
                contig_ids.remove(elem_id)
                to_be_modified_id.append(elem_id)
                with open(output_summary_info, "a") as si:
                    si.write(f"MAG {i.split('/')[-1]} has suspicious contamination contig: {elem_id}. Taxonomy: {elem_lineage_hierarchy}.\n")

            elif (EUKARYA_ID in elem_lineage_hierarchy) and (pd.isna(np.array(elem_in_crossref['markers on the contig'])[0])):
                logging.info("Contig id: {}. This is a eukaryotic contig without marker genes on it.".format(elem_id))
                contig_ids.remove(elem_id)
                to_be_modified_id.append(elem_id)
                with open(output_summary_info, "a") as si:
                    si.write(f"MAG {i.split('/')[-1]} has a eukaryotic contig without single-copy marker genes in database: {elem_id}. Taxonomy: {elem_lineage_hierarchy}.\n")

            elif (ROOT_ID in elem_lineage_hierarchy) and (pd.isna(np.array(elem_in_crossref['markers on the contig'])[0])):
                logging.info("Contig id: {}. This is a contig with ambiguous taxonomic classification and without recognizable A2K marker genes in selected lineage, discarded.\n".format(elem_id))
                contig_ids.remove(elem_id)
                to_be_modified_id.append(elem_id)
                with open(output_summary_info, "a") as si:
                    si.write(f"MAG {i.split('/')[-1]} has an unclassified contig: {elem_id}.\n")

            else:
                continue
        else:
            continue
    logging.info(f"contig composition after refinement: {','.join(contig_ids)}\n")
    logging.info(f"length equals to {len(contig_ids)}\n")
    refined_contig_ids = contig_ids
    
    # Now store new bag of contigs.
    refined_contigs_dict[i] = []
    for req in contig_comp:
        if req.id in refined_contig_ids:
            refined_contigs_dict[i].append(req)
    logging.info("Number of contigs from the refined bin {}".format(len(refined_contigs_dict[i])))
    # Length of MAG should be reported.
    new_contig_length = [len(i.seq) for i in contig_comp if i.id in contig_ids]
    logging.info(f"MAG length after refinement: {sum(new_contig_length)}")
    logging.info("-"*50)

    
for i in refined_contigs_dict.items():
    filename = output_dir_for_refined_bins + "/" + "refined_" + i[0].split("/")[-1]    
    with open(filename, "w") as refined:
        for j in i[-1]:
            logging.info(j.id)
            refined.write(">")
            refined.write(j.id)
            refined.write("\n")
            refined.write(str(j.seq))
            refined.write("\n")


cross_ref_refine_df.to_csv(snakemake.input['cross_ref'], sep="\t")



import pandas as pd
import matplotlib.pyplot as plt
import matplotlib
import seaborn as sns
import numpy as np

def compute_batch_depth(depth_Array):
    batch_depth = []
    for i in depth_Array:
        if "\t" in str(i):
            depth_list = [float(j) for j in i.split("\t")]
            batch_depth_per_contig = sum(depth_list)
            batch_depth.append(batch_depth_per_contig)
        else:
            depth_batch = float(i)
            batch_depth.append(depth_batch)
    return batch_depth

def MAG_name_simplify(DF):
    bins = list(DF.loc[:, "Contig2Bin"])
    bins_simplified = []
    for bin in bins:
        bin_feature = "_".join(["bin", *bin.split("_")[-3:]])
        bins_simplified.append(bin_feature)
    return bins_simplified

def count_and_prop(individual_bin_df):
    bin_comp_dict = dict()
    length_array_per_bin = list(individual_bin_df['contig length'])
    taxon_array_per_bin = list(individual_bin_df['Taxon per Contig'])

    # print(taxon_set)
    for i in range(0, len(taxon_array_per_bin)):
        if taxon_array_per_bin[i] in bin_comp_dict.keys():
            bin_comp_dict[taxon_array_per_bin[i]] += int(length_array_per_bin[i])
        else:
            bin_comp_dict[taxon_array_per_bin[i]] = int(length_array_per_bin[i]) # initialize dict value for new key.

    prop_list = bin_comp_dict.values()
    label_list = bin_comp_dict.keys()
    return prop_list, label_list 

def Seaborn_Violin_plot(sample_binned_df):
    fig, ax1 = plt.subplots(figsize=(35, 12))
    matplotlib.rcParams.update({'font.size': 48})
    colors = ['yellow', 'red', 'green', 'blue', 'brown','cyan','grey','goldenrod','lime','violet','indigo','coral','olive','azure', 'light pink', 'dark blue']
    color_iter = iter(colors)
    sample_binned_df['Contig2Bin'] = pd.Categorical(sample_binned_df['Contig2Bin'], [i for i in set(sample_binned_df['Contig2Bin'])])
    log10_depth_sample = np.log10(np.array(sample_binned_df['batch depth per contig']))
    sample_binned_df['log10 depth per contig'] = log10_depth_sample
    groups = sample_binned_df.groupby("Contig2Bin")
    index = 0
    for group_name, group in groups:
        sns.violinplot(x="log10 depth per contig",y="Contig2Bin", data=group, orient="h", ax=ax1, color = colors[index])
        sns.stripplot(x="log10 depth per contig", y="Contig2Bin", data=group, orient="h", jitter=True, zorder=1, size=6, color="purple")
        index += 1
    plt.yticks(fontsize=14)
    plt.xticks(fontsize=18)
    plt.xlabel("log10 depth per contig", fontsize=20)
    plt.title("Violin Plot of batch depth for each bin")
    violin_plot_path = figure_output_dir/f"{batch_name}_Contig_Depth_Violin_Plot.png"
    plt.savefig(str(violin_plot_path), dpi="figure")
    return

def marker_counter(marker_array):
    marker_count_array = []
    for i in marker_array:
        # print(i)
        if pd.isna(i):
            marker_count_array.append(0)
        else:
            marker_list = [genes for genes in i.split(",") if genes != ""]
            # print(marker_list)
            marker_count_array.append(len(marker_list))
    return marker_count_array

def compute_batch_depth(depth_Array):
    batch_depth = []
    for i in depth_Array:
        str_i = str(i)
        if "\t" in str_i:
            depth_list = [float(j) for j in i.split("\t")]
            batch_depth_per_contig = sum(depth_list)
            batch_depth.append(batch_depth_per_contig)
        else:
            depth_batch = float(i)
            batch_depth.append(depth_batch)
    return batch_depth



import os
import click

CONTIG_IDENTIFER = ">"

@click.command()

@click.option("--assembly", type=click.Path(exists=True))
@click.option("--accession", type=click.Path(exists=True))
@click.option("--fasta_file_name", type=click.STRING)


def main(assembly, accession, fasta_file_name):
    plastid_accession_id = []
    with open(accession, "r") as plastida:
        for lines in plastida:
            plastid_accession = lines.strip()
            # print(plastid_accession)
            plastid_accession_id.append(plastid_accession)
    # print(len(plastid_accession_id)) 

    n_contig = len(plastid_accession_id)
    with open(assembly, "r") as assemblyf:
        assembly_lines = assemblyf.readlines()

    # print(assembly_lines[:20])

    # make a judgement, whether there are files already there.
    if os.path.exists(fasta_file_name):
        os.remove(fasta_file_name)
    
    index = 0
    n_count = 0
    # Here, adjust the code to jump across different contig accession line, and store the sequences under the desired accession ID.
    while index < len(assembly_lines):
        if n_count == n_contig:
            break
        # By default, the first line is always an accession for the first sequence.
        accession_id = assembly_lines[index].replace(">","").split(" ")[0].strip()
        # Now, find the next index id. 
        next_accession_index = get_next_accession(assembly_lines, index)
        if accession_id in plastid_accession_id:
            sequence = "".join([seq.strip() for seq in assembly_lines[index+1:index+next_accession_index]])+"\n"
            with open(fasta_file_name, "a") as fasta:
                fasta.write(CONTIG_IDENTIFER)
                fasta.write(accession_id)
                fasta.write("\n")
                fasta.write(sequence)
                n_count += 1
        index += next_accession_index
    
    # print(os.path.exists(fasta_file_name))
    return

def get_next_accession(assembly_lines, index):
    line_skip = 0
    for i in range(index+1, len(assembly_lines)):
        if CONTIG_IDENTIFER in assembly_lines[i]:
            line_skip = i - index
            return line_skip
        else: # Make sure the sequence is already at the end.
            line_skip = len(assembly_lines)-index

    return line_skip

if __name__ == "__main__":
    main()


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import os
from os import listdir
import seaborn as sns
from pathlib import Path
import sys
from visualization_utils import *
from plotnine import ggplot, geom_point, aes, stat_smooth, facet_wrap, geom_boxplot, geom_bar, coord_flip, geom_violin
import plotnine as gg
import re


cross_reference_tsv = snakemake.input[0]
figure_output_dir = Path(snakemake.output[0])
batch_name = snakemake.params[0]
refine_bins_dir = snakemake.params['refine_bins_dir']

batch_name = batch_name.split("/")[-1]

figure_output_dir.mkdir(parents=True, exist_ok=True)

time_stamp_file = ".snakemake_timestamp"

refine_bins_list = [i for i in listdir(refine_bins_dir) if i != time_stamp_file]
if (os.stat(cross_reference_tsv).st_size == 0):
    print("Empty cross reference tsv, then nothing will be visualized.")
    sys.exit(0)
elif (len(refine_bins_list)==0):
    print("Empty bins directory after binning, process shut down.")
    sys.exit(0)

# Visualization rule will take the spreadsheet created from the last rule, and plot the statistical 
# graphics. Three graphics will be demonstrated:
#   1. contig-length distribution (a boxplot, or a histogram), to show how skewed the distribution is.
#   2. The contig length - marker number scatterplot, to show how markers distribute over this plastid assembly;
#   3. The taxonomy summarization of the assembly: how many taxa are there?
#   4. The Bin Purity plot: see how each bin's contig taxonomy homogenity?

summary_dataframe = pd.read_csv(cross_reference_tsv, sep="\t", index_col=0)

assembly_length_array = np.array(summary_dataframe['contig length'])
assembly_depth_array = np.array(summary_dataframe['contig depth'])
assembly_bin_array = list(summary_dataframe['Contig2Bin'])
contig_taxon = summary_dataframe['Taxon per Contig']
contig_2_bin = summary_dataframe['Contig2Bin']

# output the violin plot.
summary_dataframe['batch depth per contig']=compute_batch_depth(assembly_depth_array)

# depth_violin_plot(summary_dataframe, assembly_bin_array)
binned_contigs_per_batch=summary_dataframe.loc[summary_dataframe["Contig2Bin"].notna()]
simplified_binname = MAG_name_simplify(binned_contigs_per_batch)
binned_contigs_per_batch['Bin name simplified'] = simplified_binname

#store log10 contig depth.
log10_depth = np.log10(np.array(binned_contigs_per_batch['batch depth per contig']))
binned_contigs_per_batch['log10 depth per contig'] = log10_depth
violin_plot_by_bin=(
                # The plot firstly needs ggplot attribute, to select cols.
                ggplot(binned_contigs_per_batch, aes(x="Bin name simplified", y="log10 depth per contig", color="Bin name simplified"))
                + geom_violin()
                + geom_point()
                + gg.coord_flip()
                + gg.theme(figure_size=(15, 13), 
                           axis_title_x=gg.element_text(size=14), 
                           axis_title_y=gg.element_text(size=14),
                           axis_text_x=gg.element_text(size=12),
                           axis_text_y=gg.element_text(size=12)) # Theme is responsible for fontsize, legend, figsize...
).draw(show=False)

violin_plot_by_bin.savefig(f"{figure_output_dir}/LogDepth_Violin.png", dpi=300)

#2. sequence depth, gc contents and marker gene.

plt.rcParams["figure.autolayout"] = True
plt.rcParams["figure.figsize"] = [25, 36]
plt.rcParams.update({'font.size': 24})

fig1, axsc = plt.subplots(nrows=2, ncols=1)
sns.set_theme(font_scale=3)
# store count of marker genes.
markers_per_contig = binned_contigs_per_batch['markers on the contig']
marker_count = marker_counter(markers_per_contig)
binned_contigs_per_batch['marker count per contig'] = marker_count

Scatter_GC_log_depth=(
                        ggplot(binned_contigs_per_batch, aes(x="GC contents", y="log10 depth per contig", color="Bin name simplified", size = "marker count per contig"))
                            + geom_point(alpha=0.5)
                            + gg.labs(color="Bins", size="marker count per contig")
                            + gg.theme(legend_title=gg.element_text(size=12, weight='bold'), 
                                    legend_text=gg.element_text(size=8))
                            + gg.ggtitle("dot-scaled GC x LogDepth scatterplot")
                    ).draw(show=False) # Add size and colors here.
                            # Add point scale. 

Scatter_GC_log_depth.savefig(f"{figure_output_dir}/Scatter_GCLogDepth.png", dpi=300)

# # The second plot is the bar chart of the completeness and purity.
# bins = [i for i in list(set(binned_contigs_per_batch['Contig2Bin']))]
# # works at the x-label.
# bins_quality_dict = dict()
# # works as the legend.
# bins_quality_dict['Bin Completeness'] = []
# bins_quality_dict['Bin Purity'] = []
# bin_xticks = []

# completeness_pattern = re.compile(r'C(\d+)')
# purity_pattern = re.compile(r'P(\d+)')

# for i in bins:
#     bin_base_names = i.split("_")
#     prefix = "bin"
#     for j in range(len(bin_base_names)):
#         if "C" in bin_base_names[j]:
#             try: 
#                 individual_comp = int(bin_base_names[j].replace("C", ""))
#                 bins_quality_dict['Bin Completeness'].append(individual_comp)
#                 identifier = bin_base_names[j-1]
#             except ValueError:
#                 continue
#         if "P" in bin_base_names[j]:
#             individual_pur = int(bin_base_names[j].replace("P", ""))
#             bins_quality_dict['Bin Purity'].append(individual_pur)
#         else:
#             continue
#     bin_name = prefix + "." + f"{identifier}_C{individual_comp}_P{individual_pur}"
#     bin_xticks.append(bin_name)

# width = 0.2
# x=np.arange(len(bins))
# multiplier=0

# for attribute, measurement in bins_quality_dict.items():
#     # Add scatterplot of completeness against purity.

#     offset = width * multiplier
#     bin_record = axsc[1].bar(x+offset, measurement, width, label=attribute)
#     axsc[1].bar_label(bin_record, padding=3)
#     multiplier += 1

# # Add grid to axsc[0].
# axsc[0].grid(True)
# axsc[0].scatter(bins_quality_dict["Bin Completeness"], bins_quality_dict["Bin Purity"], color="red", s=120)
# axsc[0].set_title("Bin Purity vs Completeness")
# axsc[0].set_xlabel("Bin Completeness (%)")
# axsc[0].set_ylabel("Bin Purity (%)")

# axsc[1].set_ylabel("percentage")
# axsc[1].set_xlabel("bin name")
# axsc[1].set_xticks(x + width, bin_xticks, rotation=30)
# axsc[1].legend(loc='lower left', ncols=len(bins))
# axsc[1].set_ylim(50, 100)

# plt.savefig("{}/{}_bins_scatter_bar_chart.png".format(figure_output_dir, batch_name), dpi="figure")

#3. taxonomy pie chart.
bin_set = set(contig_2_bin)
bin_set
for i in bin_set:
    if not pd.isna(i):
        contig_composition = summary_dataframe.loc[summary_dataframe['Contig2Bin'] == i]
        
        # prepare the graph.
        MAGfig, MAGax = plt.subplots(figsize=(18,18))
        sns.set_theme(font_scale=2)
        plt.style.use('_mpl-gallery-nogrid')
        contig_composition_taxon_prop_bylength, taxon_labels = count_and_prop(contig_composition)[0], count_and_prop(contig_composition)[1]
        colors = plt.get_cmap('jet')(np.linspace(0.2, 0.7, len(set(contig_composition_taxon_prop_bylength))+10))
        MAGax.pie(contig_composition_taxon_prop_bylength, startangle=90, colors=colors, radius=2, wedgeprops={"linewidth": 1, "edgecolor": "white"}, center=(4,4), frame=True, autopct='%1.2f%%')
        plt.axis('equal')
        plt.legend(taxon_labels, loc="best", fontsize="29")
        plt.title("{} bin {} taxon composition".format(batch_name, i), fontsize=30)
        plt.show()
        # Hide xticks and yticks.
        plt.xticks([])
        plt.yticks([])
        plt.savefig("{}/{}_{}_taxonomy_composition.png".format(figure_output_dir, batch_name, i), dpi="figure")
    else:
        continue


import pandas as pd 
from os import listdir
import numpy as np
import sys
import logging
from Bio import SeqIO 
import urllib.request
import os
from pathlib import Path
from summarization_utils import *


# set up global variables that could work as input files.
NOT_BINNED = ""
depth_set = 0.0
CONTIG_ID = "contig id"
CONTIG_SEQ = "contig sequence"
CONTIG_DEP = "contig depth"
CONTIG_LEN = "contig length"
CONTIG_GC = "GC contents"
CONTIG_BIN = "Contig2Bin"
CONTIG_TAXON = "Taxon per Contig"
CONTIG_marker = "markers on the contig"
assemblyfasta = snakemake.params['assembly_files']
assemblydepth = snakemake.params['assembly_depth']
assembly_annot = snakemake.params['marker_gene_gff']
assembly_bin_dir = snakemake.params['bins']
taxonomy_names = snakemake.params['names_dump']
MMA_SUMMARY=snakemake.params['MMA_summary']
contig_level_annotation = os.path.join(snakemake.input['contig_level_annotation'], snakemake.params['ANNOT_FILE'])

if ((not Path(contig_level_annotation).exists() or os.stat(contig_level_annotation).st_size == 0) 
    and (not Path(assemblyfasta).exists()) 
    and (not Path(assemblydepth).exists()) 
    and (not Path(assembly_annot).exists())) :
    Path(contig_level_annotation).write_text("")
    print("No files supporting this rule to run were found, exit.\n")
    Path(snakemake.output[0]).write_text("")
    sys.exit(0)


dataset_bin_df = pd.DataFrame()
assembly_fasta_contigs = list(SeqIO.parse(assemblyfasta, "fasta"))
contig_id = []
contig_seq = []
contig_length = []
contig_gc = []
store_basic_info(contig_id, contig_seq, contig_length, contig_gc, assembly_fasta_contigs)
# formulate the length of the array for the following storage.
num_contigs = len(contig_id)

bin_array = list(np.repeat("", num_contigs))
bin_depth = list(np.repeat(0, num_contigs))
marker_per_contig = list(np.repeat("", num_contigs))
taxon_array = list(np.repeat("", num_contigs))

bin_files = listdir(assembly_bin_dir)

form_bin_array(assembly_bin_dir, bin_files, bin_array, contig_id)
form_marker_array(marker_per_contig, assembly_annot, contig_id)
form_depth_array(bin_depth, contig_id, assemblydepth)

form_contig_taxa_array(taxon_array, contig_id, taxonomy_names, contig_level_annotation)

dataset_bin_df[CONTIG_ID] = contig_id
dataset_bin_df[CONTIG_GC] = contig_gc
dataset_bin_df[CONTIG_DEP] = bin_depth
dataset_bin_df[CONTIG_LEN] = contig_length
dataset_bin_df[CONTIG_TAXON] = taxon_array
dataset_bin_df[CONTIG_marker] = marker_per_contig
dataset_bin_df[CONTIG_BIN] = bin_array
dataset_bin_df[CONTIG_SEQ] = contig_seq

dataset_bin_df.to_csv(snakemake.output[0], sep="\t")

# Here we want to add some summary to MAGs.
bin_set=set(bin_array)
medium_bins=0
high_bins=0
total_bins=len(bin_set)-1
for i in bin_set:
    if i != "":
        file_name=i.replace(".fasta","")
        essential_info=file_name.split('_')
        # breakpoint()
        # Make sure you include both possibilities.
        if ("C" in essential_info[-3]) and ("P" in essential_info[-2]):            
            completeness=int(essential_info[-3].replace("C", ""))
            purity=int(essential_info[-2].replace("P", ""))
        #else, they are in fourth and third position.
        else:
            completeness=int(essential_info[-6].replace("C", ""))
            purity=int(essential_info[-5].replace("P", ""))

        if (completeness > 70) and (completeness < 90):
            medium_bins+=1
        else:
            high_bins+=1
    else:
        continue

with open(MMA_SUMMARY, "a") as MS:
    MS.write("Total number of MAGs: {}\n".format(total_bins))
    MS.write("Total number of medium-quality (completeness: 70% to 90%): {}\n".format(medium_bins))
    MS.write("Total number of high-quality (completeness: > 90%): {}\n".format(high_bins))
    

